import os


from edge.admin.ui_types import BaseCreator
from edge.path_helpers import *
from edge.com.fs import load_json

from edge.com.net import get_lr_uds_endpoint

import edge.pipeline_helpers as helpers
from os.path import join as join_path


# noinspection PyUnusedLocal
def lr_try_get_creator_ctor(ui_entry_point, used_ui_modules, used_edge_modules, avaliable_ui_modules):
    """
    :type ui_entry_point: UiIdentity
    :type used_ui_modules: set of UiIdentity
    :type used_edge_modules: set of EdgeIdentity
    :type avaliable_ui_modules: set of UiIdentity
    :rtype: None | (str, UiIdentity, dict of UiIdentity: dict) -> BaseCreator
    """
    creator_ctor = None
    # speed
    if ui_entry_point in [Pipeline.Speed.Ui.LPR_0, Pipeline.Speed.Ui.VIDEOSOURCE_0]:

        used_ui_modules.update([Pipeline.Speed.Ui.LPR_0, Pipeline.Speed.Ui.VIDEOSOURCE_0])
        used_edge_modules.add(Pipeline.Speed.Edge.LPR_0)
        creator_ctor = LrCreator.create

    elif ui_entry_point in [Pipeline.Speed.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Speed.Ui.LPR_0, Pipeline.Speed.Ui.VIDEOSOURCE_0])
        creator_ctor = LrCreator.create

    # mv
    elif ui_entry_point in [Pipeline.Mv.Ui.LPR_0, Pipeline.Mv.Ui.VIDEOSOURCE_0]:

        used_ui_modules.update([Pipeline.Mv.Ui.LPR_0, Pipeline.Mv.Ui.VIDEOSOURCE_0])
        used_edge_modules.add(Pipeline.Mv.Edge.LPR_0)
        creator_ctor = LrCreator.create

    elif ui_entry_point in [Pipeline.Mv.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Mv.Ui.LPR_0, Pipeline.Mv.Ui.VIDEOSOURCE_0])
        creator_ctor = LrCreator.create

    # rtsp
    elif ui_entry_point in [Pipeline.Rtsp.Ui.LPR_0, Pipeline.Rtsp.Ui.VIDEOSOURCE_0]:

        used_ui_modules.update([Pipeline.Rtsp.Ui.LPR_0, Pipeline.Rtsp.Ui.VIDEOSOURCE_0])
        used_edge_modules.add(Pipeline.Rtsp.Edge.LPR_0)
        creator_ctor = LrCreator.create

    elif ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Rtsp.Ui.LPR_0, Pipeline.Rtsp.Ui.VIDEOSOURCE_0])
        creator_ctor = LrCreator.create

    return creator_ctor


class LrCreator(BaseCreator):
    def __init__(self, id_main_host_setupstore):
        """
        :type id_main_host_setupstore: str, dict, dict, str
        """
        BaseCreator.__init__(self)
        self._id_main_host_setupstore = id_main_host_setupstore

    @staticmethod
    def create(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: LrCreator
        """
        # speed
        if ui_entry_point in [Pipeline.Speed.Ui.VIDEOSOURCE_0, Pipeline.Speed.Ui.LPR_0,
                              Pipeline.Speed.Ui.PIPELINE]:

            id_main_host_setupstore = LrCreator._serialize2(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = LrCreator(id_main_host_setupstore)

        # mv
        elif ui_entry_point in [Pipeline.Mv.Ui.VIDEOSOURCE_0, Pipeline.Mv.Ui.LPR_0,
                                Pipeline.Mv.Ui.PIPELINE]:

            id_main_host_setupstore = LrCreator._serialize2(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = LrCreator(id_main_host_setupstore)

        # rtsp
        elif ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE, Pipeline.Rtsp.Ui.VIDEOSOURCE_0,
                                Pipeline.Rtsp.Ui.LPR_0]:

            id_main_host_setupstore = LrCreator._serialize2(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = LrCreator(id_main_host_setupstore)

        else:
            raise NotImplemented('pipeline not supported')

        return creator

    @staticmethod
    def _serialize2(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: LrCreator
        """
        spool_path = get_spool_dir(ui_pid)

        # speed
        if ui_entry_point.ptype == PTYPE_SPEED:
            ui_lpr_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.LPR_0]
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.VIDEOSOURCE_0]
            edge_identity = Pipeline.Speed.Edge.LPR_0
            edge_gid = edge_identity.gid
            if ui_videosource_jnode['ro']['raw-format'] == 'MONO8T':
                add_scene_react_dir = join_path(spool_path, 'mg%s-lr%s-scene' % (edge_gid, edge_gid))
            else:
                add_scene_react_dir = join_path(spool_path, 'rc%s-lr%s-scene' % (edge_gid, edge_gid))

            # raw_resolution = ui_videosource_jnode['rw']['videostream']['resolution']
            # image_wh = raw_resolution['w'], raw_resolution['h']

        # mv
        elif ui_entry_point.ptype == PTYPE_LPR_MV:
            ui_lpr_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.LPR_0]
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.VIDEOSOURCE_0]
            edge_identity = Pipeline.Mv.Edge.LPR_0
            edge_gid = edge_identity.gid

            # color images from rawcodec, mono8 from mediagrabber
            if ui_videosource_jnode['ro']['raw-format'] == 'MONO8T':
                add_scene_react_dir = join_path(spool_path, 'mg%s-lr%s-scene' % (edge_gid, edge_gid))
            else:
                add_scene_react_dir = join_path(spool_path, 'rc%s-lr%s-scene' % (edge_gid, edge_gid))

        # rtsp
        elif ui_entry_point.ptype == PTYPE_LPR_RTSP:

            ui_lpr_jnode = ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.LPR_0]
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.VIDEOSOURCE_0]
            edge_identity = Pipeline.Rtsp.Edge.LPR_0
            edge_gid = edge_identity.gid
            add_scene_react_dir = join_path(spool_path, 'rg%s-lr%s-scene' % (edge_gid, edge_gid))

        else:
            raise NotImplemented('pipeline not supported')

        raw_image_wh = helpers.get_videosource_image_wh(ui_pid, edge_gid, ui_videosource_jnode)
        image_wh = raw_image_wh if raw_image_wh else (640, 480)

        scene_event_jnode = [join_path(spool_path, 'lr%s-tr%s-scene' % (edge_gid, edge_gid))]

        # enable gc
        if ui_entry_point.ptype in [PTYPE_LPR_MV, PTYPE_SPEED]\
                and ui_videosource_jnode['rw']['videostream']['control']['profile'] == 'auto-70-150':
            scene_event_jnode.append(join_path(spool_path, 'lr%s-gc%s-scene' % (edge_gid, edge_gid)))

        plate_detector_jnode = ui_lpr_jnode['rw']['plate-detector']
        plate_detector_jnode["extra-ranges-analyse"] = ui_lpr_jnode['ro'].get('extra-ranges-analyse') or False

        main_conf_jnode = {
            "units": [
                {
                    "type": "api",
                    "name": "api",
                    "conf": {
                        "endpoint": get_lr_uds_endpoint(ui_pid)
                    }
                },
                {
                    "type": "counters",
                    "name": "counters",
                    "conf": {
                        "stat-path": join_path(spool_path, 'zabbix', 'lr%s' % edge_gid),
                        "stat-tout": 3
                    }
                },
                {
                    "type": "media",
                    "name": "media",
                    "conf": {
                        "endpoint": add_scene_react_dir
                    }
                },
                {
                    "type": "tx-track-bus",
                    "name": "tx-track-bus",
                    "conf": {
                        "store": {
                            "endpoint": join_path(spool_path, 'lr%s-lr%s-store' % (edge_gid, edge_gid)),
                            "min-file-count": 20,
                            "max-file-count": 25
                        },
                        "track": [join_path(spool_path, 'lr%s-tr%s-track' % (edge_gid, edge_gid))]
                    }
                },
                {
                    "type": "tx-scene-bus",
                    "name": "tx-scene-bus",
                    "conf": {
                        "scene": scene_event_jnode
                    }
                },
                {
                    "type": "anpr",
                    "name": "anpr",
                    "conf": {
                        "engine": {
                            "log-settings": False,
                            "license-group": int(ui_lpr_jnode['rw']['license-group']),
                            "license-count": ui_lpr_jnode['rw']['license-count'],
                            "image-tout": 250,
                            "plate-detector": plate_detector_jnode,
                            "plate-recognizer": ui_lpr_jnode['rw']['plate-recognizer'],
                            "plate-tracker": {
                                "lost-tout": ui_lpr_jnode['rw']['plate-tracker']['lost-tout'],
                                "size-min": ui_lpr_jnode['ro'].get('min-track-size', 4)
                            }
                        },
                        "stream": {
                            "wh": image_wh,
                        },
                        "postprocess": ui_lpr_jnode['ro'].get('postprocess') or 1000,
                        "activation": ui_lpr_jnode['ro'].get('activation') or "image"
                    }
                }
            ],
            "links": [
                ("anpr", "tx-scene-bus"),
                ("anpr", "media"),
                ("anpr", "api"),

                ("counters", "anpr"),
                ("counters", "tx-scene-bus"),
                ("counters", "tx-track-bus")
            ]
        }

        mmr_mode = ui_lpr_jnode['ro'].get('mmr-mode', None)
        if mmr_mode:
            main_conf_jnode['links'].append(("anpr", "mmr"))
            main_conf_jnode['links'].append(("mmr", "tx-track-bus"))
            main_conf_jnode['links'].append(("mmr", "counters"))

            mmr_jnode = {
                "conf": {
                    "mode": mmr_mode,
                    "min-conf": ui_lpr_jnode['ro'].get('mmr-min-conf', 50)
                },
                "name": "mmr",
                "type": "mmr"
            }

            nn_jnode = load_json(NN_CONF_FILEPATH)
            mmr_jnode['conf'].update(nn_jnode['ro']['networks'])

            main_conf_jnode['units'].append(mmr_jnode)
        else:
            main_conf_jnode['links'].append(("anpr", "tx-track-bus"))

        # create wiegand unit
        wiegand_conf = ui_lpr_jnode['ro'].get('wiegand-conf', None)
        if wiegand_conf:
            wiegand_jnode = {
                "type": "wiegand",
                "name": "wiegand",
                "conf": wiegand_conf
            }
            main_conf_jnode['units'].append(wiegand_jnode)
            main_conf_jnode['links'].append(("anpr", "wiegand"))
            main_conf_jnode['links'].append(("wiegand", "counters"))

        # create setupstore.conf
        edge_group = helpers.get_edge_group(ui_pid)
        edge_id = helpers.get_edge_id(edge_group, edge_identity)
        setupstore_conf_text = get_setupstore_conf_text(edge_id)

        # create host.conf
        host_conf_jnode = {
            "Core": "edge-core-lpr"
        }

        return edge_id, main_conf_jnode, host_conf_jnode, setupstore_conf_text

    def serialize(self):
        edge_id, main_conf_jnode, host_conf_jnode, setupstore_conf_text = self._id_main_host_setupstore
        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_LPR, edge_id)

        if not os.path.exists(edge_jnode_dir):
            helpers.silent_mkdir_r(edge_jnode_dir)

        helpers.silent_mkdir_r(os.path.join(edge_jnode_dir, 'unit.conf.d'))

        main_conf_filepath = join_path(edge_jnode_dir, 'unit.conf.d', 'main.conf')
        with open(main_conf_filepath, 'w') as json_file:
            ujson.dump(main_conf_jnode, json_file, indent=2)

        # create setupstore.conf
        setupstore_conf_filepath = join_path(edge_jnode_dir, 'setupstore.conf')
        with open(setupstore_conf_filepath, 'w') as text_file:
            text_file.write(setupstore_conf_text)

        # create host.conf
        host_conf_filepath = join_path(edge_jnode_dir, 'host.conf')
        with open(host_conf_filepath, 'w') as json_file:
            ujson.dump(host_conf_jnode, json_file, indent=2)


class LrDeleter:
    def __init__(self, edge_lr_dirpath):
        """
        :type edge_lr_dirpath: str
        """
        self._edge_lr_dirpath = edge_lr_dirpath

    @staticmethod
    def create(ui_pid, ui_entry_point):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :rtype: LrDeleter
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point == Pipeline.Rtsp.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Rtsp.Edge.LPR_0)
            edge_type = Pipeline.Rtsp.Edge.LPR_0.type
            deleter = LrDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Mv.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.LPR_0)
            edge_type = Pipeline.Mv.Edge.LPR_0.type
            deleter = LrDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Speed.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.LPR_0)
            edge_type = Pipeline.Speed.Edge.LPR_0.type
            deleter = LrDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        else:
            raise NotImplemented

    def run(self):
        helpers.silent_rm_r(self._edge_lr_dirpath)
