import ujson
import os

from edge.admin.ui_types import BaseCreator
from edge.path_helpers import *
from edge import eventlog

import edge.pipeline_helpers as helpers
from os.path import join as join_path


def mg_try_get_creator_ctor(ui_entry_point, used_ui_modules, used_edge_modules, avaliable_ui_modules):
    """
    :type ui_entry_point: UiIdentity
    :type used_ui_modules: set of UiIdentity
    :type used_edge_modules: set of EdgeIdentity
    :type avaliable_ui_modules: set of UiIdentity
    :rtype: None | (str, UiIdentity, dict of UiIdentity: dict) -> BaseCreator
    """
    if ui_entry_point.ptype == PTYPE_LPR_RTSP:
        return None

    ui_modules_set = set()
    if ui_entry_point.type in [UI_TYPE_VIDEOSOURCE, UI_TYPE_RED_LIGHT, UI_TYPE_PIPELINE]:
        for ui_module in avaliable_ui_modules:
            if ui_module.type in [UI_TYPE_VIDEOSOURCE, UI_TYPE_RED_LIGHT]:
                ui_modules_set.add(ui_module)

        if ui_entry_point.type != UI_TYPE_PIPELINE:
            used_edge_modules.add(EdgeIdentity(EDGE_TYPE_MEDIAGRABBER, '0'))

        used_ui_modules.update(ui_modules_set)
        creator_ctor = MgCreator.create
    else:
        return None

    return lambda arg0, arg1, arg2: creator_ctor(arg0, arg1, arg2, avaliable_ui_modules)


class MgCreator(BaseCreator):
    def __init__(self, id_main_setupstore_misc_halt):
        """
        :type id_main_setupstore_misc_halt: str, dict, str, dict
        """
        BaseCreator.__init__(self)
        self._id_main_setupstore_misc_halt = id_main_setupstore_misc_halt

    @staticmethod
    def create(ui_pid, ui_entry_point, ui_identity_to_jnode_map, avaliable_ui_modules):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :type avaliable_ui_modules: set of UiIdentity
        :rtype: RgCreator
        """

        if ui_entry_point.type in [UI_TYPE_VIDEOSOURCE, UI_TYPE_RED_LIGHT,
                                   UI_TYPE_PIPELINE]:
            id_main_setupstore_misc_halt = MgCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map,
                                                                avaliable_ui_modules)
            creator = MgCreator(id_main_setupstore_misc_halt)

        else:
            raise NotImplemented('pipeline not supported')

        return creator

    @staticmethod
    def _serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map, avaliable_ui_modules):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :type avaliable_ui_modules: set of UiIdentity
        :rtype: str, dict, str
        """
        edge_group = helpers.get_edge_group(ui_pid)
        spool_dir = get_spool_dir(ui_pid)

        if ui_entry_point.ptype == PTYPE_SPEED:
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.VIDEOSOURCE_0]
            edge_gid = Pipeline.Speed.Edge.MEDIAGRABBER_0.gid
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.MEDIAGRABBER_0)
            tx_pipes = [join_path(spool_dir, 'mg%s-ra%s-pipe' % (edge_gid, edge_gid))]

        elif ui_entry_point.ptype == PTYPE_LPR_MV:
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.VIDEOSOURCE_0]
            edge_gid = Pipeline.Mv.Edge.MEDIAGRABBER_0.gid
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.MEDIAGRABBER_0)
            tx_pipes = []
        else:
            raise NotImplemented

        ui_acquisition_jnode = ui_videosource_jnode['rw']['videostream']

        ui_sensor_jnode = ui_acquisition_jnode['resolution']

        ui_control_jnode = ui_acquisition_jnode['control']

        #
        exp_time_limits = [0, 0]
        gain_limits = [0, 0]
        gain = 0
        exp_time = 0
        mg_control_mode = 'Auto'
        if ui_control_jnode['profile'] == 'manual':
            mg_control_mode = 'Manual'

        if mg_control_mode == 'Auto':
            gain_limits = [ui_control_jnode['gain-auto-min'], ui_control_jnode['gain-auto-max']]
            exp_time_limits = [ui_control_jnode['exposure-auto-min'], ui_control_jnode['exposure-auto-max']]

        else:
            gain = ui_control_jnode['gain-manual']
            exp_time = ui_control_jnode['exposure-manual']

        lib_path = ''
        if ui_videosource_jnode['rw']['vendor'] == 'Basler':
            lib_path = '/opt/edge/libexec/mediagrabber/libmgbasler.so'

        elif ui_videosource_jnode['rw']['vendor'] == 'Teledyne DALSA':
            lib_path = '/opt/edge/libexec/mediagrabber/libdriver-dalsa.so'

        elif ui_videosource_jnode['rw']['vendor'] == 'PointGrey':
            lib_path = '/opt/edge/libexec/mediagrabber/libdriver-pointgrey.so'

        elif ui_videosource_jnode['rw']['vendor'] == 'NPK ES-Experts':
            lib_path = '/opt/edge/libexec/mediagrabber/libdriver-ese.so'

        elif ui_videosource_jnode['rw']['vendor'] == 'JAI Corporation':
            lib_path = '/opt/edge/libexec/mediagrabber/libdriver-jai.so'

        fps = ui_acquisition_jnode['fps']
        main_conf = {
            "FileApi": {
                "Store": join_path(spool_dir, 'mg%s-mg%s-store-raw' % (edge_gid, edge_gid)),
                "FileCountLow": fps * 2,
                "FileCountHigh": fps * 2 + 5,
                "SceneEvent": {
                    "Dirs": [],
                    "PixelFormat": ui_videosource_jnode['ro']['raw-format']
                },
                "HaltEvent": {
                    "SignalAbsentTout": 3000,
                    "DeviceAbsentTout": 60000
                }
            },
            "TxPipes": tx_pipes,
            "Driver": {
                "Vendor": ui_videosource_jnode['rw']['vendor'],
                "Model": ui_videosource_jnode['rw']['model'],
                "Id": ui_videosource_jnode['rw']['serial'],
                "File": lib_path,
                "ParamSet": {
                    "VideoSource": {
                        "AutoSetup": {
                            "ExpTimeLimits": exp_time_limits,
                            "Target": {
                                "Brightness": 0.50196
                            },
                            "GainLimits": gain_limits
                        },
                        "ManSetup": {
                            "Lut12": "/opt/edge/share/setupstore/edge/mediagrabber/lut12.default.conf",
                            "GainAll": gain,
                            "ExpTime": exp_time
                        },
                        "Sweep": {
                            "Profile": "MinGain",
                            "ExpTimeMode": mg_control_mode,
                            "GainMode": mg_control_mode,
                            "LutMode": "None"
                        },
                        "Acquisition": {
                            "Sensor": {
                                "Width": ui_sensor_jnode['w'],
                                "OffsetX": ui_sensor_jnode['x'],
                                "OffsetY": ui_sensor_jnode['y'],
                                "Height": ui_sensor_jnode['h']
                            },
                            "Gamma": -1,
                            "Fps": fps,
                            "Rotate180": ui_acquisition_jnode['rotate180']
                        }
                    },
                    "IoBox": [],
                    "Misc": {
                        "TransferDataSize": ui_videosource_jnode['ro']['transfer-data-size']
                    }
                }
            },
            "Monitoring": {
                "StatTout": 5,
                "StatPath": join_path(spool_dir, 'zabbix', 'vs%s' % edge_gid)
            }
        }

        if ui_videosource_jnode['ro']['raw-format'] == 'MONO8T':
            main_conf['FileApi']['SceneEvent']['Dirs'] = [
                join_path(spool_dir, 'mg%s-lr%s-scene' % (edge_gid, edge_gid)),
                join_path(spool_dir, 'mg%s-tr%s-scene' % (edge_gid, edge_gid))
            ]
            # enable mediacodecgst
            main_conf['FileApi']['SceneEvent']['Dirs'].append(
                join_path(spool_dir, 'mg%s-mcgst%s-scene' % (edge_gid, edge_gid)))

            # enable gc
            if ui_control_jnode['profile'] == 'auto-70-150':
                main_conf['FileApi']['SceneEvent']['Dirs'].append(
                    join_path(spool_dir, 'mg%s-gc%s-scene' % (edge_gid, edge_gid)))

        else:
            main_conf['FileApi']['SceneEvent']['Dirs'] = [
                join_path(spool_dir, 'mg%s-rc%s-scene' % (edge_gid, edge_gid))
            ]

        # add mg to rl path
        ui_redlight_rules_id_to_jnode_list = []
        for ui_module in avaliable_ui_modules:
            if ui_module.type == UI_TYPE_RED_LIGHT:
                ui_redlight_rules_id_to_jnode_list.append((ui_module.id, ui_identity_to_jnode_map[ui_module]))

        if len(ui_redlight_rules_id_to_jnode_list) == 1:
            ui_id, red_light_jnode = ui_redlight_rules_id_to_jnode_list[0]
            vendor = red_light_jnode['ro'].get('vendor', 'zonechecker')
            if vendor == 'zonechecker':
                main_conf['FileApi']['SceneEvent']['Dirs'].append(
                    join_path(spool_dir, 'mg%s-zc%s-scene' % (edge_gid, edge_gid))
                )

    # event-dirs from 'ro'
        try:
            extra_dirs_jnode = ui_videosource_jnode['ro']['main']['FileApi']['SceneEvent']['Dirs']
        except KeyError:
            extra_dirs_jnode = None

        if type(extra_dirs_jnode) == list:
            main_conf['FileApi']['SceneEvent']['Dirs'].extend(extra_dirs_jnode)

        elif type(extra_dirs_jnode) == unicode or type(extra_dirs_jnode) == str:
            main_conf['FileApi']['SceneEvent']['Dirs'].append(extra_dirs_jnode)

        elif extra_dirs_jnode is not None:
            eventlog.log_d('wrong ui-jnode#/ro/FileApi/SceneEvent/Dirs')

        ir_jnode = ui_videosource_jnode['ro'].get('ir', None)
        if ir_jnode is not None:
            ir = {
                "Source": ir_jnode.get('source', "ExposureActive"),
                "Mode": "Output",
                "Pin": ir_jnode['pin'],
                "Inverter": ir_jnode['sync-inverter']
            }
            main_conf['Driver']['ParamSet']['IoBox'].append(ir)

        # create setupstore.conf
        setupstore_conf_text = get_setupstore_conf_text(edge_id)

        # create haltaction.conf
        def_halt_list = [
            "0:python /opt/edge/bin/psuctl run led-camera color G blink 0HZ",
            "1:python /opt/edge/bin/psuctl run led-camera color R blink 1HZ; python /opt/edge/bin/psuctl run camera-reboot",
            "2:python /opt/edge/bin/psuctl run led-camera color R blink 0HZ"
        ]
        halt_list = ui_videosource_jnode['ro'].get('haltaction', def_halt_list)
        if halt_list:
            haltaction_conf = '\n'.join(halt_list)

        else:
            haltaction_conf = ''

        # create misc.conf
        misc_conf = ui_videosource_jnode['ro']['misc']

        return edge_id, main_conf, setupstore_conf_text, misc_conf, haltaction_conf

    def serialize(self):
        edge_id, main_conf_jnode, setupstore_conf_text, misc_conf_jnode, haltaction_conf = \
            self._id_main_setupstore_misc_halt

        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_MEDIAGRABBER, edge_id)
        if not os.path.exists(edge_jnode_dir):
            helpers.silent_mkdir_r(edge_jnode_dir)

        main_conf_abspath = join_path(edge_jnode_dir, 'main.conf')
        with open(main_conf_abspath, 'w') as main_conf:
            ujson.dump(main_conf_jnode, main_conf, indent=2)

        misc_conf_abspath = join_path(edge_jnode_dir, 'misc.conf')
        with open(misc_conf_abspath, 'w') as misc_conf:
            ujson.dump(misc_conf_jnode, misc_conf, indent=2)

        setupstore_conf_path = join_path(edge_jnode_dir, 'setupstore.conf')
        with open(setupstore_conf_path, 'w') as text_file:
            text_file.write(setupstore_conf_text)

        halt_action_path = join_path(edge_jnode_dir, 'haltaction.conf')
        with open(halt_action_path, 'w') as text_file:
            text_file.write(haltaction_conf)


class MgDeleter:
    def __init__(self, edge_mg_dirpath):
        """
        :type edge_mg_dirpath: str
        """
        self._edge_mg_dirpath = edge_mg_dirpath

    @staticmethod
    def create(ui_pid, ui_entry_point):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :rtype: MgDeleter | None
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point == Pipeline.Speed.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.MEDIAGRABBER_0)
            edge_type = Pipeline.Speed.Edge.MEDIAGRABBER_0.type

            deleter = MgDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Mv.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.MEDIAGRABBER_0)
            edge_type = Pipeline.Mv.Edge.MEDIAGRABBER_0.type

            deleter = MgDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Rtsp.Ui.PIPELINE:
            return None

        else:
            raise NotImplemented

    def run(self):
        helpers.silent_rm_r(self._edge_mg_dirpath)
