import ujson

from edge import path_helpers as ph, pipeline_helpers as oh
from edge.com import fs


_RTSP_EDGE_MODULES = [
    ph.Pipeline.Rtsp.Edge.BLOBSTORE_0,  # must be first
    ph.Pipeline.Rtsp.Edge.LPR_0,
    ph.Pipeline.Rtsp.Edge.RENDERER_0,
    ph.Pipeline.Rtsp.Edge.SPEEDCONTROL_0,
    ph.Pipeline.Rtsp.Edge.TRACKER_0,
    ph.Pipeline.Rtsp.Edge.RTSPGRABBER_0,
    ph.Pipeline.Rtsp.Edge.MEDIA_CODEC_GST_0,
    ph.Pipeline.Rtsp.Edge.TRAFFIC_LIGHT_0,
    ph.Pipeline.Rtsp.Edge.LBSAPP_0
]

_MV_EDGE_MODULES = [
    ph.Pipeline.Mv.Edge.BLOBSTORE_0,  # must be first
    ph.Pipeline.Mv.Edge.LPR_0,
    ph.Pipeline.Mv.Edge.GRAB_CONTROL_0,
    ph.Pipeline.Mv.Edge.RENDERER_0,
    ph.Pipeline.Mv.Edge.RENDERER_1,
    ph.Pipeline.Mv.Edge.SPEEDCONTROL_0,
    ph.Pipeline.Mv.Edge.TRACKER_0,
    ph.Pipeline.Mv.Edge.MEDIAGRABBER_0,
    ph.Pipeline.Mv.Edge.RTSPSERVER_0,
    ph.Pipeline.Mv.Edge.RAW_CODEC_0,
    ph.Pipeline.Mv.Edge.MEDIA_CODEC_GST_0,
    ph.Pipeline.Mv.Edge.TRAFFIC_LIGHT_0,
    ph.Pipeline.Mv.Edge.LBSAPP_0
]

_SPEED_EDGE_MODULES = [
    ph.Pipeline.Speed.Edge.BLOBSTORE_0,  # must be first
    ph.Pipeline.Speed.Edge.LPR_0,
    ph.Pipeline.Speed.Edge.GRAB_CONTROL_0,
    ph.Pipeline.Speed.Edge.RENDERER_0,
    ph.Pipeline.Speed.Edge.RENDERER_1,
    ph.Pipeline.Speed.Edge.SPEEDCONTROL_0,
    ph.Pipeline.Speed.Edge.TRACKER_0,
    ph.Pipeline.Speed.Edge.MEDIAGRABBER_0,
    ph.Pipeline.Speed.Edge.RTSPSERVER_0,
    ph.Pipeline.Speed.Edge.RADAR_0,
    ph.Pipeline.Speed.Edge.RAW_CODEC_0,
    ph.Pipeline.Speed.Edge.MEDIA_CODEC_GST_0,
    ph.Pipeline.Speed.Edge.TRAFFIC_LIGHT_0,
    ph.Pipeline.Speed.Edge.LBSAPP_0
]


def append_ss_group(ui_pid, ui_entry_point):
    """
    :type ui_pid: str
    :type ui_entry_point: UiIdentity
    """
    ss_jnode = fs.load_json(ph.SETUPSTORE_CONF_FILEPATH)
    edge_group = oh.get_edge_group(ui_pid)

    if ui_entry_point.ptype == ph.PTYPE_LPR_RTSP:
        spool_size = 512
        edge_modules = _RTSP_EDGE_MODULES

    elif ui_entry_point.ptype == ph.PTYPE_LPR_MV:
        spool_size = 512
        edge_modules = _MV_EDGE_MODULES

    elif ui_entry_point.ptype == ph.PTYPE_SPEED:
        spool_size = 512
        edge_modules = _SPEED_EDGE_MODULES

    else:
        raise NotImplemented

    new_group_node = {
        "name": "pipeline-%s" % ui_pid,
        "spool-size": spool_size,
        "modules": map(lambda edge_identity:
                       ph.get_edge_jnode_dirpath(edge_identity.type, oh.get_edge_id(edge_group, edge_identity)),
                       edge_modules)
    }

    ss_jnode['groups'].append(new_group_node)

    with open(ph.SETUPSTORE_CONF_FILEPATH, 'w') as json_file:
        ujson.dump(ss_jnode, json_file, indent=2)


def remove_ss_group(ui_pid):
    """
    :type ui_pid: str
    """
    ss_jnode = fs.load_json(ph.SETUPSTORE_CONF_FILEPATH)
    if not ss_jnode:
        raise RuntimeError

    groups_jnode = ss_jnode['groups']
    group_to_remove = filter(lambda group_jnode: group_jnode['name'] == 'pipeline-%s' % ui_pid, groups_jnode)
    if len(group_to_remove) == 1:
        groups_jnode.remove(group_to_remove[0])
        ss_jnode['groups'] = groups_jnode
        with open(ph.SETUPSTORE_CONF_FILEPATH, 'w') as json_file:
            ujson.dump(ss_jnode, json_file, indent=2)

    elif len(group_to_remove) > 1:
        raise RuntimeError('fail: rm ss group')
